:
eval 'exec $PRECISION_HOME/bin/ncp_perl -S $0 ${1+"$@"}'
    if $bogus_variable_to_keep_perl_happy;
##
## IBM Tivoli Network Manager IP Edition
##
## IBM Confidential
## OCO Source Materials
##
## 5724-S45
##
## (C) Copyright IBM Corp. 1997, 2007
##
## The source code for this program is not published or otherwise
## divested of its trade secrets, irrespective of what has
## been deposited with the U.S. Copyright Office.
#####################################################################

#####################################################################
# ITNM Data collector v1.4.0
# Update history:
# v1.4.1 - 08/06/19 - Added storm data(config and logs)
#                   - show if Java collectors are running
#                   - Added sqlite createCustomization.sql file
# v1.4.0 - 12/06/17 - Updated for 4.2
#                   - Updated for SF Case numbers
#                   - Made default collection general(-g) if no flags
#                   - Added routine to get IM logs/information
# v1.3.1 - 11/30/15 - Fixed metrics collection
# v1.3.0 - 11/11/15 - Added "en(v)iromental" collection
#                   - Added domainSummary data(domainData.txt)
#                   - Added profilingData
#                   - Added omni.dat and interfaces.arch 
#                   - Added poller metrics(>4.1)
#                   - Added network views to disco collection
#                   - Changed network views to use get_network_views
#                       if NetworkViews.DOMAINNAME.xml doesn't exist
#                   - Added scripts/sql/solid to get DNCIM
#                       customizations
#                   - Added ncp_brokerd files to log collections
#                   - Added DNCIM stitchers
#                   - Added nco_p_ncpmonitor.props|rules
#
#####################################################################
#
# This script will collect support data from an ITNM system
#
# Any/All of the information below may be collected based on
# input. 
#
# 1) $NCHOME/etc/precision
# 2) $NCHOME/log/precision
# 3) $NCHOME/precision/disco/stitchers
# 4) $NCHOME/precision/disco/agents
# 5) $NCHOME/precision/PD/core
# 6) $NCHOME/precision/amos
# 7) $NCHOME/precision/profiles/TIPProfile/logs/tnm
# 8) $NCHOME/precision/profiles/TIPProfile/etc
# 9) $TIPHOME/profiles/TIPProfile/logs
# 10) Monitors.current
# 11) Disco Cache
# 12) Model Cache(DNCIM/SolidDB/Sqlite)
# 13) Network Views xml
# 14) Polling Policies xml
# 15) Version information
# 16) OS Information
# 17) Environment Information
#
######################################################################

# use warnings;
use Data::Dumper;
use strict;
use Archive::Tar;
use Cwd;
use Net::FTP;

my $type = 'general';
my $os_version;
my $domainName;
my $cwd = getcwd;
$ENV{workingDir} = $cwd;
my $skip_logs;
my $get_cores;
my $pmr_number;
my $time=time();
$ENV{timeStamp} = $time;
my $target_dir;
my $ftp;
$ENV{scriptVersion} = "v1.4.0";

# Capture CTRL-C so we can properly cleanup
$SIG{'INT'} = sub { cleanup("interrupt"); };

# Capture ARGV to print out later
my @ARGUMENTS = @ARGV;

# Process command line arguments
while($ARGV[0]) {
   if ( $ARGV[0] =~ /-d$/) {
      $type = "discovery";
      shift;
      } elsif ( $ARGV[0] =~ /-g$/ ) {
      $type = "general";
      shift;
      } elsif ( $ARGV[0] =~ /-v$/ ) {
      $type = "environmental";
      shift;
      } elsif ( $ARGV[0] =~ /-u$/ ) {
      $type = "gui";
      shift;
      } elsif ( $ARGV[0] =~ /-r$/ ) {
      $type = "rca";
      shift;
      } elsif ( $ARGV[0] =~ /-p$/ ) {
      $type = "polling";
      shift;
      } elsif ( $ARGV[0] =~ /-i$/ ) {
      $type = "install";
      shift;
      } elsif ( $ARGV[0] =~ /-t$/ ) {
      if (! defined($ENV{ITMHOME})) {
         $ENV{ITMHOME} = get_itm_home();
         }
      $type = "itm";
      shift;
      } elsif ( $ARGV[0] =~ /-e$/ ) {
      $type = "report";
      shift;
      } elsif ( $ARGV[0] =~ /-dc$/ ) {
      $type = "disco_cache";
      shift;
      } elsif ( $ARGV[0] =~ /-mc$/ ) {
      $type = "model_cache";
      shift;
      } elsif ( $ARGV[0] =~ /-x$/ ) {
      $skip_logs = 1;
      shift;
      } elsif ( $ARGV[0] =~ /-c$/ ) {
      $get_cores = 1;
      shift;
      } elsif ( $ARGV[0] =~ /-f$/ ) {
      $ftp = 1;
      shift;
      } elsif ( $ARGV[0] =~ /-o$/ ) {
      $target_dir = $ARGV[1];
      shift;
      shift;
      } elsif ( $ARGV[0] =~ /-pmr$/ ) {
      # Check for a proper PMR number, alert and discard if bad, but continue
      if ($ARGV[1] =~ /^\d{5}[\W]\w{3}[\W]\w{3}$/ || $ARGV[1] =~ /^TS[\d]{8}/i ) {
         $pmr_number = $ARGV[1];
         $pmr_number =~ s/\W/./g;
         } else {
         status_message("PMR/Case Number doesn't match requirements and will not be used.");
         status_message("Format should be: 11111,111,111 or TS0xxxxxxx.");
         }
      shift;
      shift;
      } elsif ( $ARGV[0] =~ /-domain$/ ) {
      $ENV{domainName} = $ARGV[1];
      $domainName = $ENV{domainName};
      shift;
      shift;
      } else {
      show_usage(); 
      exit;
      }
   }

# Throw a status message if being run interactive
if (defined($type)) {
   # used for tooling
   $ENV{type} = $type;
   status_message("Collecting OS, Environment, and version information."); 
   }

# Make sure NCHOME is set, exit if not
if ($ENV{NCHOME} =~ /^\s*$/) {
   if (-d "/opt/IBM/tivoli/netcool") {
      # If this is install, and we have no $NCHOME, we'll request it
      if ($type =~ /install/) {
         my $nchome = "";
         print "Please enter NCHOME directory(/opt/IBM/tivoli/netcool by default): ";
         $nchome = <STDIN>;
         chomp($nchome);
         until ( -d "$nchome") {
            status_message("$nchome is not a valid directory. Try again.");
            $nchome = <STDIN>;
            chomp($nchome);
            }
         $ENV{NCHOME} = $nchome;
         } else {
         status_message("\$NCHOME variable must be set, source your environment and retry.");
         exit 255;
         }
      } else {
      $ENV{NCHOME} = "/opt/IBM/tivoli/netcool";
      }
   }

# We need a temporary directory in the service dir, create service if it doesn't exist
if ( ! -d "$ENV{NCHOME}/service" ) {
   mkdir ("$ENV{NCHOME}/service") || cleanup("Unable to create $ENV{NCHOME}/service($!)");
   }

# Create a TEMP dir for us to dump information into
$ENV{collectorDir} = "$ENV{NCHOME}/service/data-$time";
mkdir($ENV{collectorDir}) || cleanup("Unable to create $ENV{NCHOME}/service/data-$time($!)");

# We need a few other directories for tooling
mkdir("$ENV{collectorDir}/autopdzip") || cleanup("Unable to create $ENV{collectorDir}/autopdzip");
mkdir("$ENV{collectorDir}/autopdzip/autopd") || cleanup("Unable to create $ENV{collectorDir}/autopdzip/autopd");

# We need to create a full path to TIPHOME and Components home if they exist
if ( -d "$ENV{NCHOME}/../tipv2" ) {
   chdir("$ENV{NCHOME}/../tipv2");
   $ENV{tipHome} = getcwd;
   chdir($cwd);
   }

if ( -d "$ENV{NCHOME}/../tipv2Components" ) {
   chdir("$ENV{NCHOME}/../tipv2Components");
   $ENV{componentHome} = getcwd;
   chdir($cwd);
   }

# Base Information Collection
# OS, Version, and configuration files

my @dfdata;
my @cpudata;
my @memdata;

# First let's determine the OS and grab Environment info
if ($^O =~ /aix/) {
   $os_version = qx(oslevel -r);
   @cpudata = qx(lscfg | grep proc);
   @memdata = qx(lsattr -El sys0 -a realmem);
   } elsif ( $^O =~ /linux/ ) {
   if ( -e "/etc/redhat-release" ) {
      open(FILE,"/etc/redhat-release");
      $os_version = <FILE>;
      close(FILE);
      } elsif ( -e "/etc/SuSE-release") {
      open(FILE,"/etc/SuSE-release");
      $os_version = <FILE>;
      close(FILE);
      } else {
      $os_version = "Unable to find release file. Unknown Distro.";
      }
   @cpudata = qx(cat /proc/cpuinfo | grep -i 'model name');
   @memdata = qx(free -m);
   } elsif ( $^O =~ /solaris/ ) {
   open(FILE,"/etc/release");
   $os_version = <FILE>; 
   close(FILE);
   @cpudata = qx(psrinfo);
   @memdata = qx(prtconf | grep Memory);
   }

@dfdata = qx(df -k);

# Store cpu and memory data
if (@cpudata) {
   write_to_file("cpu.txt",\@cpudata);
   }
if (@memdata) {
   write_to_file("memory.txt",\@memdata);
   }
if (@dfdata) {
   write_to_file("df.txt",\@dfdata);
   }

$ENV{OSVersion} = $os_version;   # used for tooling
chomp($ENV{OSVersion});
my @data = ("$os_version");
write_to_file("os_info.txt",\@data);
write_to_file("cli_arguments.txt",\@ARGUMENTS);

# Store version info for tooling
if ( -e "$ENV{NCHOME}/precision/bin/ncp_ctrl") {
   $ENV{itnmVersion} = qx($ENV{NCHOME}/precision/bin/ncp_ctrl -version | grep Build | sed -e 's/\).*/\)/');
   chomp($ENV{itnmVersion});
   my @data = split / /,$ENV{itnmVersion};
   $ENV{itnmRawVersion} = $data[5];
   }

# Tooling information
if (defined($type)) {
   write_tooling_info($type);
   } else {
   write_tooling_info();
   }

# Want to get ncp_testfix.pl output if it exists
if ( -f "$ENV{NCHOME}/service/ncp_testfix.pl" ) {
   @data = qx(ncp_perl $ENV{NCHOME}/service/ncp_testfix.pl -list);
   write_to_file("fix_list.txt",\@data);
   }

# If this a core discovery(polling,discovery,rca,general) we will
# grab the itnm_status as well

if (defined ($type) && $type =~ /disco|polling|rca|general|environmental|gui/ ) {
   my @status_data;
   my @status;
   if ( -f "$ENV{NCHOME}/precision/bin/itnm_status" ) {
      status_message("Collecting daemon status");
      @status_data;
      @status = qx($ENV{NCHOME}/precision/bin/itnm_status);
      push(@status_data, @status);
      push(@status_data, "\n\n----------- Process Information(ps -ef | grep ncp) -----------\n");
      }
   my @ps = qx(ps -ef | grep ncp | grep -v grep);
   push(@status_data, @ps);
   push(@status_data, "\n\n----------- Running Java Collectors(ps -ef | grep propsFile) -----------\n");
   @ps = ();
   @ps = qx(ps -ef | grep propsFile | grep -v grep);
   push(@status_data, @ps);
   write_to_file("itnm_status.txt",\@status_data);
   }

# Now grab all the ENV/ulimit info:
my @env;
my $string;

foreach my $key (keys %ENV) {
   $string = $key." = ". $ENV{$key}."\n";
   push(@env,$string);
   }

push (@env, "\nUlimits\n-----\n\n");
my @ulimit = qx(bash -c "ulimit -a");
push(@env, @ulimit);
write_to_file("environment_info.txt",\@env);

my @file_list;

# We will always get the etc/precision, conf.key, omni.dat, interfaces.arch, listIU.log, .db2sqllib and itnm.cfg
push(@file_list,"$ENV{NCHOME}/etc/precision/");

my @always_get_these = ("$ENV{NCHOME}/etc/security/keys/conf.key",
                        "$ENV{NCHOME}/precision/.db2sqllib",
                        "$ENV{NCHOME}/etc/itnm.cfg",
                        "$ENV{NCHOME}/etc/omni.dat",
                        "$ENV{NCHOME}/etc/interfaces.linux2x86",
                        "$ENV{NCHOME}/etc/interfaces.aix4",
                        "$ENV{NCHOME}/etc/interfaces.aix5",
                        "$ENV{NCHOME}/log/listIU.log",
                        "$ENV{NCHOME}/etc/interfaces.solaris2");

push(@file_list, @always_get_these);

# Skip log files if flag set or install collection
if (! defined($skip_logs) || defined($type)) {
   if (defined($type) && $type !~ /model_cache|disco_cache|itm|report/) {

      # Get the log directory in case of custom logdir in CtrlServices
      my $logdir = "$ENV{NCHOME}/log/precision/";
      my $CtrlServices;

      # make sure we have a CtrlServices file, and if so, get the log dir
      if ( -e "$ENV{NCHOME}/etc/precision/CtrlServices.$domainName.cfg" ) {
         $CtrlServices = "$ENV{NCHOME}/etc/precision/CtrlServices.$domainName.cfg";
         } elsif ( -e "$ENV{NCHOME}/etc/precision/CtrlServices.cfg" ) {
         $CtrlServices = "$ENV{NCHOME}/etc/precision/CtrlServices.cfg";
         }

      # open the file and get the first non-comment line containing logdir
      if ( -f "$CtrlServices" ) {
         open(CTRL,"< $CtrlServices");
         while (<CTRL>) {
            if ($_ =~ /\[.*logdir\",\s+\"([^\"]+)\"/ ) {
               my $tempDir = $1;
               $tempDir =~ s/\$NCHOME\///;
   
               # Make sure the directory exists, and if so change logdir
               if ( -d "$ENV{NCHOME}/$tempDir") {
                  $logdir = "$ENV{NCHOME}/$tempDir";
                  $ENV{customLogs} = $logdir;
                  last;
                  }
               }
            }
         close(CTRL);
         }

      if ( $logdir =~ /\/$/ ) {
         push(@file_list,"$logdir");
         } else {
         push(@file_list,"$logdir/");
         }
      }
   }

# Get core files if flag set
if (defined($get_cores)) {
   my $cores = "$ENV{NCHOME}/precision/PD/core/";
   if (-d "$cores") {
      my @core_data = files_from_dir($cores);
      push(@file_list,@core_data);
      }
   }

# Call top-level collection target as needed
my @files_to_collect;

if (defined($type)) {
   if ($type =~ /discovery/) {
      @files_to_collect = discovery_collection();
      } elsif ($type =~ /general/) {
      @files_to_collect = general_collection();
      } elsif ($type =~ /environmental/) {
      @files_to_collect = environmental_collection();
      } elsif ($type =~ /gui/) {
      @files_to_collect = gui_collection();
      } elsif ($type =~ /rca/) {
      @files_to_collect = rca_collection();
      } elsif ($type =~ /polling/) {
      @files_to_collect = polling_collection();
      } elsif ($type =~ /install/) {
      @files_to_collect = install_collection();
      } elsif ($type =~ /model_cache/) {
      @files_to_collect = model_cache();
      } elsif ($type =~ /disco_cache/) {
      @files_to_collect = disco_cache();
      } elsif ($type =~ /itm/) {
      @files_to_collect = itm_collection();
      } elsif ($type =~ /report/) {
      @files_to_collect = report_collection();
      }
   }

push(@file_list,@files_to_collect);

# process files and directories to create specific list
# of files to store(zip)

my $member;
my $file;
my $filename;
my @to_collect;

foreach my $member (@file_list) {
   chomp($member);

   # replace the templatename with the domain name and add the file 
   if (defined ($domainName) && $member =~ /##DOMAIN##/) {
      $member =~ s/##DOMAIN##/$domainName/;
      }

   # Directory ?
   if ($member =~ /\/$/) {

      # This is a directory, make sure it exists, then
      # add each file to the collection

      if (-d "$member") {
         my @in_dir = glob("$member*");
         foreach my $target (@in_dir) {
            next if $target =~ /.lck$/;
            if (-f "$target") {
               if ($target =~ /log\/precision/ || $target =~ /$ENV{customLogs}/) {
                  # if we are in log/precsion we only want .log, .trace and ncp_brokerd files
                  if (defined ($domainName)) {
                     if ($target =~ /$domainName.log$/ || $target =~ /$domainName.trace$/ || $target =~ /ncp_broker/) {
                        push(@to_collect,$target);
                        }
                     } elsif (! defined ($type)) {
                     # only grab them without domain if script is run without flags(no $type)
                     push(@to_collect,$target);
                     }
                  } else {
                  if ($target =~ /cognos/ && ($target =~ /\.log$/ || $target =~ /\.trace$/ )) {
                     # stat_the_file just checks to see file is less than a week old 
                     if (stat_the_file($target)) {
                        push(@to_collect, $target);
                        }
                     } else {
                     push(@to_collect,$target);
                     }
                  }
               } else {

               # If its in the uninst directory we want to grab all the files 
               if ($target =~ /_uninst/) {
                  my @inventory = files_from_dir($target);
                  push(@to_collect, @inventory);
                  } 
               }
            }
         }

      } else {

      # If the filename has a wildcard, we need to glob the info, else just add it
      if ($member =~ /\*/) {
         my @files = glob("$member");
         push(@to_collect, @files);
         } else {
         if (-f "$member") {
            push(@to_collect, $member);
            }
         }
      }
   }


# OK, lets cd to our collector directory and get all the files
chdir($ENV{collectorDir});
my @files = files_from_dir($ENV{collectorDir});

# remove unnecessary leading directories
s/$ENV{collectorDir}\/// for @files;

# Now we can start building the tar file
status_message("Tarring and compressing file. Please be patient..");

if (! defined($target_dir)) {
   $target_dir = "/tmp";
   }

# Create the filename, put the PMR number first if available. This will
# allow it to be processed properly by ECURep
if (defined ($pmr_number) && defined($type)) {
   $filename = $target_dir."/".$pmr_number."_".$type."_".$time.".tar";
   } else {
   if (defined ($type)) { 
      $filename = $target_dir."/itnmdata_".$type."_".$time.".tar";
      } else {
      $filename = $target_dir."/itnmdata_".$time.".tar";
      }
   }


tar_the_file($ENV{collectorDir},$filename,\@files);

# grab files from the common directory and leave out as much of 
# the path as possible

my @nchome;
my @tipv2;
my @components;
my @others;
my @home;
my @itmhome;

foreach $file (@to_collect) {
    if ($file =~ /^$ENV{NCHOME}/) {
       $file =~ s/$ENV{NCHOME}\///;
       push(@nchome, $file);
       } elsif ($file =~ /^$ENV{tipHome}\//) {
       $file =~ s/$ENV{tipHome}\///;
       push(@tipv2, $file);
       } elsif ($file =~ /^$ENV{componentHome}/) {
       $file =~ s/$ENV{componentHome}\///;
       push(@components, $file);
       } elsif ($file =~ /^$ENV{ITMHOME}/) {
       $file =~ s/$ENV{ITMHOME}\///;
       push(@itmhome, $file);
       } elsif ($file =~ /^$ENV{HOME}/) {
       $file =~ s/$ENV{HOME}\///;
       push(@home, $file);
       } else {
       push(@others, $file);
       }
    }

if (@nchome) {
   tar_the_file($ENV{NCHOME},$filename,\@nchome);
   }

if (@components) {
   tar_the_file($ENV{componentHome},$filename,\@components);
   }

if (@tipv2) {
   tar_the_file($ENV{tipHome},$filename,\@tipv2);
   }
     
if (@home) {
   tar_the_file($ENV{HOME},$filename,\@home);
   }

if (@itmhome) {
   tar_the_file($ENV{ITMHOME},$filename,\@itmhome);
   }
     
if (@others) {
   tar_the_file($ENV{workingDir},$filename,\@others);
   }

# Compress the target file
my $gzip = qx(which gzip);
chomp($gzip);

if (defined($gzip)) {
   my @toss = qx($gzip $filename);
   $filename = $filename.".gz";
   }

# Send out the files
if (defined($ftp) && defined($pmr_number)) {
   my $ftp_return = ftp_the_file($filename);
   if ($ftp_return != 0) {
      status_message("Unable to send the file($filename) to IBM. Please send it manually.");
      }
   } else {
   status_message("$filename was created. Please provide this file to support.");
   }

cleanup("complete");

exit;

# Subs for collection targets:
# 1) Discovery
# 2) Polling
# 3) GUI
# 4) RCA
# 5) Install
#
# These call the data targets, to add data, add to or create
# additional data targets, collection targets should be upper 
# level collections

sub discovery_collection {
   add_disco_cache();
   add_db_access();
   add_domain_summary();
   my @logs;
   my @stitchers_and_agents = add_disco_stitchers_agents_and_collectors();
   my @model = add_model_cache();
   my @profilingData = add_profiling_data();
   my $network_views = add_network_views();
   my $aoc = add_aoc_files();
   my @customSchema = add_schema_customizations();
   push (@logs, @customSchema);
   push (@logs, @stitchers_and_agents);
   push (@logs, @model);
   push (@logs, @profilingData);
   push (@logs, $network_views);
   push (@logs, $aoc);
   return (@logs);
   }

sub environmental_collection {
   add_db_access();
   add_domain_summary();
   my @logs;
   my @probe_data = add_probe_data();
   my @profilingData = add_profiling_data();
   my @stitchers_and_agents = add_disco_stitchers_agents_and_collectors();
   my @model = add_model_cache();
   my $network_views = add_network_views();
   my $aoc = add_aoc_files();
   my $gateway = add_gateway_rules();

   if ($ENV{itnmRawVersion} =~ /^4./) {
      my @poller_metrics = add_poller_metrics();
      push(@logs, @poller_metrics);
      }

   if (defined ($ENV{domainName})) {
      add_monitors_current();
      my $policies = add_polling_policies();
      push (@logs, $policies);
      }

   push (@logs, @profilingData);
   push (@logs, @stitchers_and_agents);
   push (@logs, @model);
   push (@logs, @probe_data);
   push (@logs, $network_views);
   push (@logs, $gateway);
   push (@logs, $aoc);
   return (@logs);
   }

sub polling_collection {
   add_db_access();
   my @logs;
   my @probe_data = add_probe_data();
   my @profilingData = add_profiling_data();
   my @model = add_model_cache();
   my $network_views = add_network_views();
   my $aoc = add_aoc_files();

   if ($ENV{itnmRawVersion} =~ /^4./) {
      my @poller_metrics = add_poller_metrics();
      push(@logs, @poller_metrics);
      }

   if (defined ($ENV{domainName})) {
      add_monitors_current();
      my $policies = add_polling_policies();
      push (@logs, $policies);
      }

   my $poll_logs = add_polling_logs();
   my @customSchema = add_schema_customizations();
   my @stormData = add_storm_logs_and_config();
   push (@logs, @stormData);
   push (@logs, @customSchema);
   push (@logs, @probe_data);
   push (@logs, @profilingData);
   push (@logs, @model);
   push (@logs, $network_views);
   push (@logs, $aoc);
   push (@logs, $poll_logs);
   return(@logs);
   }

sub general_collection {
   add_db_access();
   if ($ENV{itnmRawVersion} =~ /[4-9].[2-9]/) { 
      get_installationmanager_data();
      }
   my @logs;
   my $network_views = add_network_views();
   my $policies = add_polling_policies();
   push (@logs, $network_views);
   push (@logs,$policies);
   return(@logs);
   }

sub gui_collection {
   my @logs;
   my @tip_logs = add_tip_logs_and_config();
   my $network_views = add_network_views();
   push (@logs, @tip_logs);
   push (@logs, $network_views);
   return(@logs);
   }

sub report_collection {
   my @logs;
   my @tip_logs = add_tip_logs_and_config();
   my @tcr_logs = add_tcr_logs();
   push (@logs, @tip_logs);
   push (@logs, @tcr_logs);
   return(@logs);
   }

sub rca_collection {
   my @logs;
   my @model = add_model_cache();
   my @probe_data = add_probe_data();
   my $gateway = add_gateway_rules();
   my $aoc = add_aoc_files();
   my $policies = add_polling_policies();
   push (@logs,@model);
   push (@logs, @probe_data);
   push (@logs,$gateway);
   push (@logs,$aoc);
   push (@logs,$policies);
   return(@logs);
   }

sub install_collection {
   my @logs;
   my @tip_logs = add_tip_logs_and_config();
   my @install_logs = add_install_logs();
   add_db_access();
   if ($ENV{itnmRawVersion} =~ /4.[2-9]/) { 
      get_installationmanager_data();
      }
   push (@logs, @tip_logs);
   push (@logs, @install_logs);
   return(@logs);
   }

sub itm_collection {
   my @logs;
   my @itm_logs = add_itm_logs();
   push (@logs,@itm_logs);
   return(@logs);
   }
      
sub disco_cache {
   add_disco_cache();
   my @logs;
   my @customSchema = add_schema_customizations();
   push (@logs, @customSchema);
   return(@logs);
   }

sub model_cache {
   add_db_access();
   my @logs;
   my @model = add_model_cache();
   my $aoc = add_aoc_files();
   my @customSchema = add_schema_customizations();
   push (@logs, @customSchema);
   push (@logs, @model);
   push (@logs, $aoc);
   return(@logs);
   }


# These subs are the actual data targets either add additional
# data to an existing target, or if it can be used by multiple
# collection targets, create a new data target and then add
# that to the appropriate collection targets
# NOTE: due to earlier logic, all directories should end in /

sub add_aoc_files {
   my $logs;
   $logs = "$ENV{NCHOME}/precision/aoc/";
   return($logs);
   }

sub add_schema_customizations {
   my @logs;
   push(@logs, "$ENV{NCHOME}/precision/scripts/sql/sqlite/createCustomization.sql");
   # Added to retreive DNCIM customizations(05/09/15 PS)
   push(@logs, "$ENV{NCHOME}/precision/scripts/sql/solid/");
   return(@logs);
   }

sub add_disco_stitchers_agents_and_collectors {  
   my @logs;
   push(@logs, "$ENV{NCHOME}/precision/disco/stitchers/");
   push(@logs, "$ENV{NCHOME}/precision/disco/stitchers/DNCIM/");
   push(@logs, "$ENV{NCHOME}/precision/disco/agents/");
   push(@logs, "$ENV{NCHOME}/precision/disco/agents/perlAgents/");

   # Get collectors
   my @perlCollectors = glob("$ENV{NCHOME}/precision/collectors/perlCollectors/*");
   foreach my $dir (@perlCollectors) {
      push(@logs, "$dir\/\*");
      }
   
   if ( $ENV{itnmRawVersion} =~ /^4./ )	{
      my @javaCollectors = glob("$ENV{NCHOME}/precision/collectors/javaCollectors/*");
      foreach my $dir (@javaCollectors) {
         push(@logs, "$dir\/\*");
         }
      }
   return(@logs);
   }

sub add_gateway_rules {  
   my $logs;
   $logs =  "$ENV{NCHOME}/precision/eventGateway/stitchers/";
   return($logs);
   }

sub add_storm_logs_and_config {
   my @logs;
   push(@logs, "$ENV{NCHOME}/log/precision/storm/*.log");
   push(@logs, "$ENV{NCHOME}/precision/storm/conf/");
   return(@logs);
   }

sub add_tip_logs_and_config {  
   my @logs;
   push(@logs, "$ENV{tipHome}/profiles/TIPProfile/logs/");
   push(@logs, "$ENV{tipHome}/profiles/TIPProfile/logs/server1/");
   push(@logs, "$ENV{NCHOME}/precision/profiles/TIPProfile/logs/tnm/");
   push(@logs, "$ENV{NCHOME}/precision/profiles/TIPProfile/etc/tnm/");
   push(@logs, "$ENV{NCHOME}/precision/profiles/TIPProfile/etc/tnm/autoprovision/");
   return(@logs);
   }

sub add_model_cache {  
   my @logs;
   if ($ENV{itnmRawVersion} =~ /^4./) {
      push(@logs, "$ENV{NCHOME}/var/precision/Store.Cache.ncimCache.*.##DOMAIN##");
      push(@logs, "$ENV{NCHOME}/precision/embeddedDb/solid/ncp_disco.##DOMAIN##/");
      push(@logs, "$ENV{NCHOME}/precision/embeddedDb/sqlite/ncp_disco.##DOMAIN##/");
      } else {
      push(@logs, "$ENV{NCHOME}/var/precision/Store.Cache.kernel.activeModel.##DOMAIN##");
      }
   return(@logs);
   }

sub add_network_views {  
   my $logs;

   my $target = "$ENV{NCHOME}/var/precision/NetworkViews.$ENV{domainName}.xml";

   if ( -e $target ) {
      $logs =  "$ENV{NCHOME}/var/precision/NetworkViews.##DOMAIN##.xml";
      } else {
      # The NetworkViews.DOMAIN.xml doesn't exist or we are > 4.1, instead
      # we'll run get_network_views.pl

      $target = "$ENV{collectorDir}/NetworkViews.$ENV{domainName}.xml";
      if ( -e "$ENV{NCHOME}/precision/scripts/perl/scripts/get_network_views.pl" ) {
         qx(ncp_perl $ENV{NCHOME}/precision/scripts/perl/scripts/get_network_views.pl -export -domain $ENV{domainName} -file "$target");
         } elsif ( -e "$ENV{NCHOME}/precision/scripts/upgrade/ITNMExportNetworkViews.pl" ) {
         # qx command above after getting ncim credentials
         get_ncim_credentials() if (! exists($ENV{NCIMusername}));
         qx(ncp_perl $ENV{NCHOME}/precision/scripts/upgrade/ITNMExportNetworkViews.pl -server $ENV{NCIMserver} -host $ENV{NCIMhostname} -username $ENV{NCIMusername} -password "$ENV{NCIMpassword}" -port $ENV{NCIMportnum} -export);
         # now copy the file to $ENV{collectorDir}
         }
      $logs = "$ENV{collectorDir}/NetworkViews.$ENV{domainName}.xml";
      }
   return($logs);
   }

sub add_polling_policies {  
   my $logs;
   $logs = "$ENV{NCHOME}/var/precision/MonitorPolicies.##DOMAIN##.xml";
   return($logs);
   }

sub add_polling_logs {
   my $logs;
   $logs = "$ENV{NCHOME}/precision/platform/java/lib/ncp_topoviz/logs/tnm/";
   return($logs);
   }

sub add_install_logs {
   my @logs;
   push (@logs, "$ENV{HOME}/IA-ITNM-Install*.log");
   push (@logs, "$ENV{HOME}/TIP*.log");
   push (@logs, $ENV{HOME}."/.acsi_".$ENV{LOGNAME}."/logs/netcool/");
   push (@logs, "$ENV{NCHOME}/log/install/");
   push (@logs, "$ENV{tipHome}/logs/");
   push (@logs, "$ENV{tipHome}/_uninst/");
   push (@logs, "/usr/ibm/common/acsi/logs/root/");
   push (@logs, "$ENV{NCHOME}/omnibus/log/");
   return(@logs);
   }

sub add_tcr_logs {
   my @logs;
   push (@logs, "$ENV{componentHome}/TCRComponent/logs/");
   push (@logs, "$ENV{componentHome}/TCRComponent/cognos/logs/");
   push (@logs, "$ENV{componentHome}/BIRTExtension/logs/");
   push (@logs, "$ENV{componentHome}/ESSServer/logs/");
   return(@logs);
   }

sub add_itm_logs {
   my @logs;
   push(@logs, "$ENV{ITMHOME}/logs/");
   push(@logs, "$ENV{ITMHOME}/config/");
   push(@logs, "$ENV{ITMHOME}/aix526/np/bin/");
   push(@logs, "$ENV{ITMHOME}/linux2x86/np/bin/");
   push(@logs, "$ENV{ITMHOME}/solaris/np/bin/");
   return(@logs);
   }

sub add_profiling_data {
   my @logs;
   push(@logs, "$ENV{NCHOME}/var/precision/Disco.Cache.disco.profilingData.$ENV{domainName}");
   push(@logs, "$ENV{NCHOME}/var/precision/Model.Cache.model.profilingData.$ENV{domainName}");
   return(@logs);
   }

sub add_poller_metrics {
   my @logs;
   push(@logs, "$ENV{NCHOME}/log/precision/ncp_poller.*.##DOMAIN##.metrics");
   return(@logs);
   }

sub add_db_access {
   status_message("Check DB access.");
   my @dbdata;
   my $domainName = $ENV{domainName}; 
   my $script = "$ENV{NCHOME}/precision/scripts/perl/scripts/ncp_db_access.pl";
   if ( -e $script ) {
      @dbdata = qx(ncp_perl $script -domain $domainName);
      }
   my $filename = "ncp_db_access.txt";
   write_to_file($filename,\@dbdata);
   }

sub add_monitors_current {  
   status_message("Collecting polling information. This could take a few minutes.");

   # First we need to get all the poller names
   # with those, we can query the monitors.current tables
   # for each poller

   my $get_it=0;
   my @pollers;

   # Open and read in the appropriate CtrlServices file:
   my $domainName = $ENV{domainName}; 
   my $dir = $ENV{collectorDir};
   my $prefix = "$ENV{NCHOME}/etc/precision";

   if (defined $domainName && -e "$prefix/CtrlServices.$domainName.cfg") {
      $filename = $prefix."/CtrlServices.".$domainName.".cfg";
      } else {
      $filename = $prefix."/CtrlServices.cfg";
      }
  
   open(FILE,$filename) || cleanup("Unable to open $filename($!).");
   my @data = <FILE>;
   close(FILE);

   # grab the poller names
   foreach my $line (@data) {
      my $poller_name = "";
      if ($line =~ /\"ncp_poller\"/) {
         $get_it = 1;
         }

      # look for the appropriate line and make sure its not a comment
      if ($get_it == 1 && $line =~ /\[.*\-domain/ && $line !~ /^\s*\/\// && $line !~ /^\s*#/) {
         $line =~ /.*"-name"[^"]*"([^"]*)"/;
         $poller_name = $1;
         if (defined ($poller_name)) {
            push(@pollers,$poller_name);
            } else {
            push(@pollers,"default");
            }
         } elsif ( $get_it == 1 && $line =~ /\);/) {
         $get_it = 0;
         }
      }

   my %passed_var;
   foreach my $poller (@pollers) {
      $passed_var{query} = "select * from monitors.current\;";
      $passed_var{poller} = $poller;
      $passed_var{type} = "SnmpPoller";

      my @returned = oql_query(%passed_var);

      $filename = "$poller.txt";
      write_to_file($filename, \@returned);
      }
   }

sub add_probe_data {
   my @logs = glob("$ENV{NCHOME}/probes/*/nco_p_ncpmonitor.[pr]*");
   return(@logs);
   }

sub add_domain_summary {
   my $query = "select * from ncim.domainSummary ds inner join ncim.domainMgr dm on dm.domainMgrId = ds.domainMgrId;";
   my @domain_data = ncim_query("$query");
   write_to_file("domainData.txt", \@domain_data);
   }

sub oql_query {
   # We need a target(disco/model/SnmpPoller)
   # We need the domain name
   # We need the query 
   # We may need poller name (for SnmpPoller if not default)
   
   my %data = @_;
   my $poller;

   my $domain = $ENV{domainName};
   if (defined($data{poller})) {
      $poller = $data{poller};
      }
   my $type = $data{type};
   my $query = $data{query};

   my @oql_data;
   if (defined($poller) && $poller !~ /^default$/) {
      @oql_data = qx(ncp_oql -domain $domain -service $type -poller "$poller" -query "$query");
      } else {
      @oql_data = qx(ncp_oql -domain $domain -service $type -query "$query");
      }
     
   return(@oql_data);
   }

sub ncim_query {
   # expecting ncim query
   get_ncim_credentials() if (! exists($ENV{NCIMusername}));
   my @ncim_data;

   if ($ENV{NCIMusername} !~ /FAILED/) {
      my $query = $_[0];
      @ncim_data = qx(ncp_oql -service NCIM -domain $ENV{domainName} -username $ENV{NCIMusername} -password $ENV{NCIMpassword} -query "$_[0]");
      }
   return (@ncim_data);
   }

sub get_ncim_credentials {
   $ENV{NCIMusername} = "FAILED";
   $ENV{NCIMpassword} = "FAILED";

   # Find the dblogins file, or return notfound
   my $dblogins = "$ENV{NCHOME}/etc/precision/DbLogins.".$ENV{domainName}.".cfg";
   if ( ! -e $dblogins ) {
      $dblogins = "$ENV{NCHOME}/etc/precision/DbLogins.cfg"; 
      if ( ! -e $dblogins) {
         return;
         }
      } 

   open (my $dbloginfile, '<', $dblogins) or return;
   my @dbdata = <$dbloginfile>;
   close $dbloginfile;

   my $start = "none";
   my @top;
   my %data;
   my %logindata;
   my @bottom;

   foreach my $line (@dbdata) {
   
      # We need to pull out the full stanzas and 
      # the markers below help parse out each stanza
      chomp($line);
      if ($line =~ /insert into config\.dbserver/i) {
         $start = "top";
         next;
         }
      if ($line =~ /values/i) {
         $start = "bottom";
         next;
         }
      if ($start =~ /bottom/ && $line =~ /\)\;/) {
         $start = "done";
         }
   
      if ($start =~ /top/ && $line =~ /\s+(m_[^,]*),?/) {
         push(@top,$1);
         next;
         }
      if ($start =~ /bottom/ && $line =~ /([^\\\(].*)/) {
         # We need to clean up the characters we dont want
         my $hold = $1;
         $hold =~ s/['",\s]//g;
         push(@bottom,$hold);
         next;
         }
   
      if ($start =~ /done/) {
   
         # Stanza is complete, need to create the data hash with the 
         # data from the two arrays setting the values for each 
         # respectively(0 = 0, 1 = 1);
   
         for (my $i=0;$i<=$#top;$i++) {
            $logindata{$top[$i]} = $bottom[$i];
            }
   
         # Need only work with the NCIM DbId
         if ($logindata{m_DbId} == "NCIM") {
            $ENV{NCIMdbname} = $logindata{m_DbName};
            $ENV{NCIMserver} = $logindata{m_Server};
            $ENV{NCIMhostname} = $logindata{m_Hostname};
            $ENV{NCIMportnum} = $logindata{m_PortNum};
            $ENV{NCIMusername} = $logindata{m_Username};
            my $encrypted = $logindata{m_Password};
   
            my @pwdata;
   
            # if the password isn't encrypted, just set it and return
            if ($logindata{m_EncryptedPwd} == 1) {
               @pwdata = qx(ncp_crypt -decrypt -password "$logindata{m_Password}");
               } else {
               $ENV{NCIMpassword} = $logindata{m_Password};
               return;
               }
   
            foreach my $bit (@pwdata) {
               next if ($bit !~ /\@/);
               # this will match the unecrypted password
               $bit =~ /\'([^\']*)\'/;
               $ENV{NCIMpassword} = $1;
               last;
               }
    
            last;
            } else {
            # Reset all vars for next loop
            @top = ();
            @bottom = ();
            %data = ();
            $start = "none";
            }
         }
      }
   return;
   }

sub add_disco_cache {  
   status_message("Building disco cache. This may take a few minutes.");
   my $domainName = $ENV{domainName}; 
   my $dir = $ENV{collectorDir};
   my $pwd = getcwd;
   if (defined($domainName)) {
      my $script = "$ENV{NCHOME}/precision/scripts/perl/scripts/GetDiscoCache.pl";
      my @toss = qx(cd $dir;ncp_perl $script -domain $domainName 2>&1 > /dev/null;cd $pwd);
      }
   }

sub files_from_dir {
   my $dir = $_[0];
   my @file_data = qx(find $dir -type f);
   chomp(@file_data);
   return (@file_data);
   }

sub write_to_file {
   my $filename = $_[0];
   my @data = @{$_[1]};
   
   my $target = $ENV{collectorDir}."/$filename";
   
   open (FILE, ">$target");
   foreach my $line (@data) {
      if ($line =~ /\n/) {
         print FILE $line;
      	 } else {
         print FILE "$line\n";
      	 }
      }
   close(FILE);
   }

sub cleanup {
   my $done = $_[0];
   if ($done !~ /complete/) {
      if ($done =~ /interrupt/) {
         status_message("Process interrupted, cleaning up.");
         } else {
         status_message("FAIL: $done");
         }
      chdir($ENV{workingDir});
      system("rm -rf $ENV{collectorDir}") if $ENV{collectorDir} =~ /service\/data/;
      status_message("Cleanup complete.");
      exit;
      }
   chdir($ENV{workingDir});
   # only remove the directory if its the correct file path
   system("rm -rf $ENV{collectorDir}") if $ENV{collectorDir} =~ /service\/data/;
   }

sub status_message {
   if (! defined $ENV{quiet}) {
      print $_[0]."\n";
      }
   }

sub stat_the_file {

   # A lot of times we have very old logs and traces in cognos, there is really no 
   # reason to collect these files. This will keep us from collecting any log or 
   # trace files that have not been modified in a week

   my @file_data = stat($_[0]);
   my $week_old = time() - 604800;
   if ($file_data[9] >= $week_old) {
      return(1);
      } else {
      return(0);
      }
   }
   
sub ftp_the_file {
   my $message;
   my $file = $_[0];
   my $ftp=Net::FTP->new("ftp.ecurep.ibm.com", Debug => 0);
   $ftp->login("anonymous",'-anonymous@') or return(1); 
   $ftp->cwd("/toibm/tivoli") or return(1);
   $ftp->binary();
   $ftp->put($file) or return(1);
   $ftp->quit;
   status_message("The file($file) was successfully uploaded to IBM.");
   return(0);
   }

sub tar_the_file {
   my $flags;
   my $directory = $_[0];             # directory we want to tar files from
   my $target = $_[1];                # tar filename
   my $files = join(" ",@{$_[2]});    # list of files
   #escape any parenthesis
   $files =~ s/\(/\\\(/g;
   $files =~ s/\)/\\\)/g;
  
   # Find tar
   my $tar = qx(which tar);
   chomp($tar);

   if ( -e "$target" ) {
      # Then the file exists, we want to add to it
      $flags = "-rf";
      } else {
      # we want to create the file
      $flags = "-cf";
      }

   my $cwd = $ENV{workingDir};
   my $command = "$tar $flags $target $files 2>&1 > /dev/null";

   my @toss = qx(cd $directory;$command;cd $cwd);
   return;
   }

sub get_itm_home {
   my $itmhome;
   if (! -d "/opt/IBM/ITM") {

      # ITMHOME is not defined and /opt/IBM/ITM does not exist, let's get it 
      print "Unable to find \$ITMHOME. Please provide path to ITM(/opt/IBM/ITM by default): ";
      $itmhome = <STDIN>;
      chomp($itmhome);

      while (! -d "$itmhome") {
         print "$itmhome is not a valid direcotry, please try again: ";
         $itmhome = <STDIN>;
         chomp($itmhome);
         }

      } else {
      # /opt/IBM/ITM exists, we'll assume its the ITMHOME directory and use it
      $itmhome = "/opt/IBM/ITM";
      } 
   return($itmhome);
   } 

sub get_installationmanager_data {
   my $im_dir = qx(awk -F= '/im_install_location/ {print \$2}' "$ENV{NCHOME}/etc/itnm.cfg");
   chomp($im_dir);
   if ($im_dir) {
      my @im_data = qx($im_dir/tools/imcl listInstalledPackages -long);
      write_to_file("listInstalledPackages.txt",\@im_data);
      @im_data = qx($im_dir/tools/imcl exportInstallData $ENV{collectorDir}/im_logs.zip);
      }
   }

sub write_tooling_info {
   push(my @data,"<?xml version='1.0' encoding='UTF-8'?>\n");
   push(@data ,"<collectionEnvironmentInfo pluginTaxonomyId='SSSHRK' toolName='itnm_data' toolVersion='0.0.1' ");
   push(@data ,"xmlns='http://www.ibm.com/autopd/collectionEnvironment' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' ");
   push(@data, "xsi:schemaLocation='http://www.ibm.com/autopd/collectionEnvironment ../autoPD-collection-env.xsd' />\n");
   write_to_file("environment-v2.xml",\@data);
   write_to_file("autopdzip/autopd/autopd-collection-environment.xml",\@data);
   my $time;
   if (defined($ENV{timeStamp})) {
      $time = scalar(localtime($ENV{timeStamp}));
      } else {
      $time = time();
      }

   push(my @stamp_data,"--------------------- ITNM Data Collector($ENV{scriptVersion}) ---------------------\n");
   if (defined ($ENV{type})) {
      push(@stamp_data,"$ENV{type} data collected by $ENV{LOGNAME} on $time\n");
      } else {
      push(@stamp_data,"Data collected by $ENV{LOGNAME} on $time\n");
      }

   if (defined($ENV{domainName})) {
      push(@stamp_data, "Domain: $ENV{domainName}\n");
      }

   if (defined($ENV{itnmVersion})) {
      push(@stamp_data,"Version:\n$ENV{itnmVersion}\n");
      }

   if (defined($ENV{OSVersion})) {
      push(@stamp_data,"Running on:\n$ENV{OSVersion}\n");
      }

   push(@stamp_data,"-----------------------------------------------------------------\n");
   write_to_file("autopdzip/autopd/pmrstamp.info",\@stamp_data);
   }

sub show_usage {
   print "Usage: ncp_perl itnm_data.pl -domain <domainName> [[-i],[-d],[-g],[-u],[-r],[-p],[-mc],[-dc],[-e],[-t]]\n";
   print "Collection types(use one): (i)nstall, (d)iscovery, (g)eneral, g(u)i, (r)ca, (p)olling, \n";
   print "\t\t\t   (mc) model cache, (dc) disco cache, r(e)port, i(t)m\n";
   print "\nCollection Modifiers:\n";
   print "\t\t[-pmr] PMR/Case Number(format XXXXX,XXX,XXX or TSxxxxxxxxx)\n";
   print "\t\t[-o] output directory(/tmp by default)\n";
   print "\t\t[-x] skip \$NCHOME/log/precision files\n";
   print "\t\t[-c] get core files\n";
   print "\t\t[-f] FTP the file to IBM(-pmr required)\n";
   }
  
